from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .market_order_transaction import MarketOrderTransaction
from .order_cancel_transaction import OrderCancelTransaction
from .order_fill_transaction import OrderFillTransaction
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="ClosePositionResponse200")


@dataclasses.dataclass
class ClosePositionResponse200:
    """Attributes:
    long_order_create_transaction (Optional[MarketOrderTransaction]): A MarketOrderTransaction represents the
        creation of a Market Order in the user's account. A Market Order is an Order that is filled immediately at the
        current market price.
        Market Orders can be specialized when they are created to accomplish a specific task: to close a Trade, to
        closeout a Position or to particiate in in a Margin closeout.
    long_order_fill_transaction (Optional[OrderFillTransaction]): An OrderFillTransaction represents the filling
        of an Order in the client's Account.
    long_order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction represents the
        cancellation of an Order in the client's Account.
    short_order_create_transaction (Optional[MarketOrderTransaction]): A MarketOrderTransaction represents the
        creation of a Market Order in the user's account. A Market Order is an Order that is filled immediately at the
        current market price.
        Market Orders can be specialized when they are created to accomplish a specific task: to close a Trade, to
        closeout a Position or to particiate in in a Margin closeout.
    short_order_fill_transaction (Optional[OrderFillTransaction]): An OrderFillTransaction represents the
        filling of an Order in the client's Account.
    short_order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction represents the
        cancellation of an Order in the client's Account.
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    long_order_create_transaction: Optional["MarketOrderTransaction"]
    long_order_fill_transaction: Optional["OrderFillTransaction"]
    long_order_cancel_transaction: Optional["OrderCancelTransaction"]
    short_order_create_transaction: Optional["MarketOrderTransaction"]
    short_order_fill_transaction: Optional["OrderFillTransaction"]
    short_order_cancel_transaction: Optional["OrderCancelTransaction"]
    related_transaction_i_ds: Optional[List[str]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order_cancel_transaction import OrderCancelTransaction
        from .market_order_transaction import MarketOrderTransaction
        from .order_fill_transaction import OrderFillTransaction

        d = src_dict.copy()
        _long_order_create_transaction = d.pop("longOrderCreateTransaction", None)
        long_order_create_transaction: Optional[MarketOrderTransaction]
        if _long_order_create_transaction is None:
            long_order_create_transaction = None
        else:
            long_order_create_transaction = MarketOrderTransaction.from_dict(
                _long_order_create_transaction
            )
        _long_order_fill_transaction = d.pop("longOrderFillTransaction", None)
        long_order_fill_transaction: Optional[OrderFillTransaction]
        if _long_order_fill_transaction is None:
            long_order_fill_transaction = None
        else:
            long_order_fill_transaction = OrderFillTransaction.from_dict(
                _long_order_fill_transaction
            )
        _long_order_cancel_transaction = d.pop("longOrderCancelTransaction", None)
        long_order_cancel_transaction: Optional[OrderCancelTransaction]
        if _long_order_cancel_transaction is None:
            long_order_cancel_transaction = None
        else:
            long_order_cancel_transaction = OrderCancelTransaction.from_dict(
                _long_order_cancel_transaction
            )
        _short_order_create_transaction = d.pop("shortOrderCreateTransaction", None)
        short_order_create_transaction: Optional[MarketOrderTransaction]
        if _short_order_create_transaction is None:
            short_order_create_transaction = None
        else:
            short_order_create_transaction = MarketOrderTransaction.from_dict(
                _short_order_create_transaction
            )
        _short_order_fill_transaction = d.pop("shortOrderFillTransaction", None)
        short_order_fill_transaction: Optional[OrderFillTransaction]
        if _short_order_fill_transaction is None:
            short_order_fill_transaction = None
        else:
            short_order_fill_transaction = OrderFillTransaction.from_dict(
                _short_order_fill_transaction
            )
        _short_order_cancel_transaction = d.pop("shortOrderCancelTransaction", None)
        short_order_cancel_transaction: Optional[OrderCancelTransaction]
        if _short_order_cancel_transaction is None:
            short_order_cancel_transaction = None
        else:
            short_order_cancel_transaction = OrderCancelTransaction.from_dict(
                _short_order_cancel_transaction
            )
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        last_transaction_id = d.pop("lastTransactionID", None)
        close_position_response_200 = cls(
            long_order_create_transaction=long_order_create_transaction,
            long_order_fill_transaction=long_order_fill_transaction,
            long_order_cancel_transaction=long_order_cancel_transaction,
            short_order_create_transaction=short_order_create_transaction,
            short_order_fill_transaction=short_order_fill_transaction,
            short_order_cancel_transaction=short_order_cancel_transaction,
            related_transaction_i_ds=related_transaction_i_ds,
            last_transaction_id=last_transaction_id,
        )
        close_position_response_200.additional_properties = d
        return close_position_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_request import OrderRequest
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="ReplaceOrderBody")


@dataclasses.dataclass
class ReplaceOrderBody:
    """Attributes:
    order (Optional[OrderRequest]): The base Order specification used when requesting that an Order be created.
        Each specific Order-type extends this definition."""

    order: Optional["OrderRequest"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order_request import OrderRequest

        d = src_dict.copy()
        _order = d.pop("order", None)
        order: Optional[OrderRequest]
        if _order is None:
            order = None
        else:
            order = OrderRequest.from_dict(_order)
        replace_order_body = cls(order=order)
        replace_order_body.additional_properties = d
        return replace_order_body

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .trade_summary_state import TradeSummaryState
from .trade_summary_state import check_trade_summary_state
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="TradeSummary")


@dataclasses.dataclass
class TradeSummary:
    """The summary of a Trade within an Account. This representation does not provide the full details of the Trade's
    dependent Orders.

        Attributes:
            id (Optional[str]): The Trade's identifier, unique within the Trade's Account.
            instrument (Optional[str]): The Trade's Instrument.
            price (Optional[str]): The execution price of the Trade.
            open_time (Optional[str]): The date/time when the Trade was opened.
            state (Optional[TradeSummaryState]): The current state of the Trade.
            initial_units (Optional[str]): The initial size of the Trade. Negative values indicate a short Trade, and
                positive values indicate a long Trade.
            initial_margin_required (Optional[str]): The margin required at the time the Trade was created. Note, this
                is the 'pure' margin required, it is not the 'effective' margin used that factors in the trade risk if a GSLO is
                attached to the trade.
            current_units (Optional[str]): The number of units currently open for the Trade. This value is reduced to
                0.0 as the Trade is closed.
            realized_pl (Optional[str]): The total profit/loss realized on the closed portion of the Trade.
            unrealized_pl (Optional[str]): The unrealized profit/loss on the open portion of the Trade.
            margin_used (Optional[str]): Margin currently used by the Trade.
            average_close_price (Optional[str]): The average closing price of the Trade. Only present if the Trade has
                been closed or reduced at least once.
            closing_transaction_i_ds (Optional[List[str]]): The IDs of the Transactions that have closed portions of
                this Trade.
            financing (Optional[str]): The financing paid/collected for this Trade.
            close_time (Optional[str]): The date/time when the Trade was fully closed. Only provided for Trades whose
                state is CLOSED.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            take_profit_order_id (Optional[str]): ID of the Trade's Take Profit Order, only provided if such an Order
                exists.
            stop_loss_order_id (Optional[str]): ID of the Trade's Stop Loss Order, only provided if such an Order
                exists.
            trailing_stop_loss_order_id (Optional[str]): ID of the Trade's Trailing Stop Loss Order, only provided if
                such an Order exists."""

    id: Optional[str]
    instrument: Optional[str]
    price: Optional[str]
    open_time: Optional[str]
    state: Optional[TradeSummaryState]
    initial_units: Optional[str]
    initial_margin_required: Optional[str]
    current_units: Optional[str]
    realized_pl: Optional[str]
    unrealized_pl: Optional[str]
    margin_used: Optional[str]
    average_close_price: Optional[str]
    closing_transaction_i_ds: Optional[List[str]]
    financing: Optional[str]
    close_time: Optional[str]
    client_extensions: Optional["ClientExtensions"]
    take_profit_order_id: Optional[str]
    stop_loss_order_id: Optional[str]
    trailing_stop_loss_order_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", None)
        instrument = d.pop("instrument", None)
        price = d.pop("price", None)
        open_time = d.pop("openTime", None)
        _state = d.pop("state", None)
        state: Optional[TradeSummaryState]
        if _state is None:
            state = None
        else:
            state = check_trade_summary_state(_state)
        initial_units = d.pop("initialUnits", None)
        initial_margin_required = d.pop("initialMarginRequired", None)
        current_units = d.pop("currentUnits", None)
        realized_pl = d.pop("realizedPL", None)
        unrealized_pl = d.pop("unrealizedPL", None)
        margin_used = d.pop("marginUsed", None)
        average_close_price = d.pop("averageClosePrice", None)
        closing_transaction_i_ds = cast(List[str], d.pop("closingTransactionIDs", None))
        financing = d.pop("financing", None)
        close_time = d.pop("closeTime", None)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        take_profit_order_id = d.pop("takeProfitOrderID", None)
        stop_loss_order_id = d.pop("stopLossOrderID", None)
        trailing_stop_loss_order_id = d.pop("trailingStopLossOrderID", None)
        trade_summary = cls(
            id=id,
            instrument=instrument,
            price=price,
            open_time=open_time,
            state=state,
            initial_units=initial_units,
            initial_margin_required=initial_margin_required,
            current_units=current_units,
            realized_pl=realized_pl,
            unrealized_pl=unrealized_pl,
            margin_used=margin_used,
            average_close_price=average_close_price,
            closing_transaction_i_ds=closing_transaction_i_ds,
            financing=financing,
            close_time=close_time,
            client_extensions=client_extensions,
            take_profit_order_id=take_profit_order_id,
            stop_loss_order_id=stop_loss_order_id,
            trailing_stop_loss_order_id=trailing_stop_loss_order_id,
        )
        trade_summary.additional_properties = d
        return trade_summary

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.stop_order_reject_transaction_position_fill import (
    check_stop_order_reject_transaction_position_fill,
)
from ..models.stop_order_reject_transaction_position_fill import (
    StopOrderRejectTransactionPositionFill,
)
from ..models.stop_order_reject_transaction_reason import (
    check_stop_order_reject_transaction_reason,
)
from ..models.stop_order_reject_transaction_reason import (
    StopOrderRejectTransactionReason,
)
from ..models.stop_order_reject_transaction_reject_reason import (
    check_stop_order_reject_transaction_reject_reason,
)
from ..models.stop_order_reject_transaction_reject_reason import (
    StopOrderRejectTransactionRejectReason,
)
from ..models.stop_order_reject_transaction_time_in_force import (
    check_stop_order_reject_transaction_time_in_force,
)
from ..models.stop_order_reject_transaction_time_in_force import (
    StopOrderRejectTransactionTimeInForce,
)
from ..models.stop_order_reject_transaction_trigger_condition import (
    check_stop_order_reject_transaction_trigger_condition,
)
from ..models.stop_order_reject_transaction_trigger_condition import (
    StopOrderRejectTransactionTriggerCondition,
)
from ..models.stop_order_reject_transaction_type import (
    check_stop_order_reject_transaction_type,
)
from ..models.stop_order_reject_transaction_type import StopOrderRejectTransactionType
from typing import Union

if TYPE_CHECKING:
    from ..models.take_profit_details import TakeProfitDetails
    from ..models.trailing_stop_loss_details import TrailingStopLossDetails
    from ..models.client_extensions import ClientExtensions
    from ..models.stop_loss_details import StopLossDetails


T = TypeVar("T", bound="StopOrderRejectTransaction")


@_attrs_define
class StopOrderRejectTransaction:
    """A StopOrderRejectTransaction represents the rejection of the creation of a Stop Order.

    Attributes:
        id (Union[Unset, str]): The Transaction's Identifier.
        time (Union[Unset, str]): The date/time when the Transaction was created.
        user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
        batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
        type (Union[Unset, StopOrderRejectTransactionType]): The Type of the Transaction. Always set to
            "STOP_ORDER_REJECT" in a StopOrderRejectTransaction.
        instrument (Union[Unset, str]): The Stop Order's Instrument.
        units (Union[Unset, str]): The quantity requested to be filled by the Stop Order. A posititive number of units
            results in a long Order, and a negative number of units results in a short Order.
        price (Union[Unset, str]): The price threshold specified for the Stop Order. The Stop Order will only be filled
            by a market price that is equal to or worse than this price.
        price_bound (Union[Unset, str]): The worst market price that may be used to fill this Stop Order. If the market
            gaps and crosses through both the price and the priceBound, the Stop Order will be cancelled instead of being
            filled.
        time_in_force (Union[Unset, StopOrderRejectTransactionTimeInForce]): The time-in-force requested for the Stop
            Order.
        gtd_time (Union[Unset, str]): The date/time when the Stop Order will be cancelled if its timeInForce is "GTD".
        position_fill (Union[Unset, StopOrderRejectTransactionPositionFill]): Specification of how Positions in the
            Account are modified when the Order is filled.
        trigger_condition (Union[Unset, StopOrderRejectTransactionTriggerCondition]): Specification of which price
            component should be used when determining if an Order should be triggered and filled. This allows Orders to be
            triggered based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy)
            price depending on the desired behaviour. Orders are always filled using their default price component.
            This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
            condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
            transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
            is set to the default value when indicating the distance from an Order's trigger price, and will always provide
            the default trigger condition when creating or modifying an Order.
            A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
            value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
            a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
        reason (Union[Unset, StopOrderRejectTransactionReason]): The reason that the Stop Order was initiated
        client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        take_profit_on_fill (Union[Unset, TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Union[Unset, StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Union[Unset, TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        intended_replaces_order_id (Union[Unset, str]): The ID of the Order that this Order was intended to replace
            (only provided if this Order was intended to replace an existing Order).
        reject_reason (Union[Unset, StopOrderRejectTransactionRejectReason]): The reason that the Reject Transaction was
            created
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, StopOrderRejectTransactionType] = UNSET
    instrument: Union[Unset, str] = UNSET
    units: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    price_bound: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, StopOrderRejectTransactionTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    position_fill: Union[Unset, StopOrderRejectTransactionPositionFill] = UNSET
    trigger_condition: Union[Unset, StopOrderRejectTransactionTriggerCondition] = UNSET
    reason: Union[Unset, StopOrderRejectTransactionReason] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    take_profit_on_fill: Union[Unset, "TakeProfitDetails"] = UNSET
    stop_loss_on_fill: Union[Unset, "StopLossDetails"] = UNSET
    trailing_stop_loss_on_fill: Union[Unset, "TrailingStopLossDetails"] = UNSET
    trade_client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    intended_replaces_order_id: Union[Unset, str] = UNSET
    reject_reason: Union[Unset, StopOrderRejectTransactionRejectReason] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        instrument = self.instrument

        units = self.units

        price = self.price

        price_bound = self.price_bound

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        position_fill: Union[Unset, str] = UNSET
        if not isinstance(self.position_fill, Unset):
            position_fill = self.position_fill

        trigger_condition: Union[Unset, str] = UNSET
        if not isinstance(self.trigger_condition, Unset):
            trigger_condition = self.trigger_condition

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        take_profit_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_on_fill, Unset):
            take_profit_on_fill = self.take_profit_on_fill.to_dict()

        stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_on_fill, Unset):
            stop_loss_on_fill = self.stop_loss_on_fill.to_dict()

        trailing_stop_loss_on_fill: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = self.trailing_stop_loss_on_fill.to_dict()

        trade_client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_client_extensions, Unset):
            trade_client_extensions = self.trade_client_extensions.to_dict()

        intended_replaces_order_id = self.intended_replaces_order_id

        reject_reason: Union[Unset, str] = UNSET
        if not isinstance(self.reject_reason, Unset):
            reject_reason = self.reject_reason

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if units is not UNSET:
            field_dict["units"] = units
        if price is not UNSET:
            field_dict["price"] = price
        if price_bound is not UNSET:
            field_dict["priceBound"] = price_bound
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if position_fill is not UNSET:
            field_dict["positionFill"] = position_fill
        if trigger_condition is not UNSET:
            field_dict["triggerCondition"] = trigger_condition
        if reason is not UNSET:
            field_dict["reason"] = reason
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if take_profit_on_fill is not UNSET:
            field_dict["takeProfitOnFill"] = take_profit_on_fill
        if stop_loss_on_fill is not UNSET:
            field_dict["stopLossOnFill"] = stop_loss_on_fill
        if trailing_stop_loss_on_fill is not UNSET:
            field_dict["trailingStopLossOnFill"] = trailing_stop_loss_on_fill
        if trade_client_extensions is not UNSET:
            field_dict["tradeClientExtensions"] = trade_client_extensions
        if intended_replaces_order_id is not UNSET:
            field_dict["intendedReplacesOrderID"] = intended_replaces_order_id
        if reject_reason is not UNSET:
            field_dict["rejectReason"] = reject_reason

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.take_profit_details import TakeProfitDetails
        from ..models.trailing_stop_loss_details import TrailingStopLossDetails
        from ..models.client_extensions import ClientExtensions
        from ..models.stop_loss_details import StopLossDetails

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, StopOrderRejectTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_stop_order_reject_transaction_type(_type)

        instrument = d.pop("instrument", UNSET)

        units = d.pop("units", UNSET)

        price = d.pop("price", UNSET)

        price_bound = d.pop("priceBound", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, StopOrderRejectTransactionTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_stop_order_reject_transaction_time_in_force(
                _time_in_force
            )

        gtd_time = d.pop("gtdTime", UNSET)

        _position_fill = d.pop("positionFill", UNSET)
        position_fill: Union[Unset, StopOrderRejectTransactionPositionFill]
        if isinstance(_position_fill, Unset):
            position_fill = UNSET
        else:
            position_fill = check_stop_order_reject_transaction_position_fill(
                _position_fill
            )

        _trigger_condition = d.pop("triggerCondition", UNSET)
        trigger_condition: Union[Unset, StopOrderRejectTransactionTriggerCondition]
        if isinstance(_trigger_condition, Unset):
            trigger_condition = UNSET
        else:
            trigger_condition = check_stop_order_reject_transaction_trigger_condition(
                _trigger_condition
            )

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, StopOrderRejectTransactionReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = check_stop_order_reject_transaction_reason(_reason)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        _take_profit_on_fill = d.pop("takeProfitOnFill", UNSET)
        take_profit_on_fill: Union[Unset, TakeProfitDetails]
        if isinstance(_take_profit_on_fill, Unset):
            take_profit_on_fill = UNSET
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)

        _stop_loss_on_fill = d.pop("stopLossOnFill", UNSET)
        stop_loss_on_fill: Union[Unset, StopLossDetails]
        if isinstance(_stop_loss_on_fill, Unset):
            stop_loss_on_fill = UNSET
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)

        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", UNSET)
        trailing_stop_loss_on_fill: Union[Unset, TrailingStopLossDetails]
        if isinstance(_trailing_stop_loss_on_fill, Unset):
            trailing_stop_loss_on_fill = UNSET
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )

        _trade_client_extensions = d.pop("tradeClientExtensions", UNSET)
        trade_client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_trade_client_extensions, Unset):
            trade_client_extensions = UNSET
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )

        intended_replaces_order_id = d.pop("intendedReplacesOrderID", UNSET)

        _reject_reason = d.pop("rejectReason", UNSET)
        reject_reason: Union[Unset, StopOrderRejectTransactionRejectReason]
        if isinstance(_reject_reason, Unset):
            reject_reason = UNSET
        else:
            reject_reason = check_stop_order_reject_transaction_reject_reason(
                _reject_reason
            )

        stop_order_reject_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            price_bound=price_bound,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            position_fill=position_fill,
            trigger_condition=trigger_condition,
            reason=reason,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            intended_replaces_order_id=intended_replaces_order_id,
            reject_reason=reject_reason,
        )

        stop_order_reject_transaction.additional_properties = d
        return stop_order_reject_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .account_summary import AccountSummary
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetAccountSummaryResponse200")


@dataclasses.dataclass
class GetAccountSummaryResponse200:
    """Attributes:
    account (Optional[AccountSummary]): A summary representation of a client's Account. The AccountSummary does
        not provide to full specification of pending Orders, open Trades and Positions.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account."""

    account: Optional["AccountSummary"]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .account_summary import AccountSummary

        d = src_dict.copy()
        _account = d.pop("account", None)
        account: Optional[AccountSummary]
        if _account is None:
            account = None
        else:
            account = AccountSummary.from_dict(_account)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_account_summary_response_200 = cls(
            account=account, last_transaction_id=last_transaction_id
        )
        get_account_summary_response_200.additional_properties = d
        return get_account_summary_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

StopOrderTimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]

STOP_ORDER_TIME_IN_FORCE_VALUES: Set[StopOrderTimeInForce] = {
    "FOK",
    "GFD",
    "GTC",
    "GTD",
    "IOC",
}


def check_stop_order_time_in_force(value: str) -> StopOrderTimeInForce:
    if value in STOP_ORDER_TIME_IN_FORCE_VALUES:
        return cast(StopOrderTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_ORDER_TIME_IN_FORCE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .stop_loss_order_request_time_in_force import StopLossOrderRequestTimeInForce
from .stop_loss_order_request_time_in_force import (
    check_stop_loss_order_request_time_in_force,
)
from .stop_loss_order_request_trigger_condition import (
    StopLossOrderRequestTriggerCondition,
)
from .stop_loss_order_request_trigger_condition import (
    check_stop_loss_order_request_trigger_condition,
)
from .stop_loss_order_request_type import StopLossOrderRequestType
from .stop_loss_order_request_type import check_stop_loss_order_request_type
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="StopLossOrderRequest")


@dataclasses.dataclass
class StopLossOrderRequest:
    """A StopLossOrderRequest specifies the parameters that may be set when creating a Stop Loss Order. Only one of the
    price and distance fields may be specified.

        Attributes:
            type (Optional[StopLossOrderRequestType]): The type of the Order to Create. Must be set to "STOP_LOSS" when
                creating a Stop Loss Order.
            trade_id (Optional[str]): The ID of the Trade to close when the price threshold is breached.
            client_trade_id (Optional[str]): The client ID of the Trade to be closed when the price threshold is
                breached.
            price (Optional[str]): The price threshold specified for the Stop Loss Order. If the guaranteed flag is
                false, the associated Trade will be closed by a market price that is equal to or worse than this threshold. If
                the flag is true the associated Trade will be closed at this price.
            distance (Optional[str]): Specifies the distance (in price units) from the Account's current price to use as
                the Stop Loss Order price. If the Trade is short the Instrument's bid price is used, and for long Trades the ask
                is used.
            time_in_force (Optional[StopLossOrderRequestTimeInForce]): The time-in-force requested for the StopLoss
                Order. Restricted to "GTC", "GFD" and "GTD" for StopLoss Orders.
            gtd_time (Optional[str]): The date/time when the StopLoss Order will be cancelled if its timeInForce is
                "GTD".
            trigger_condition (Optional[StopLossOrderRequestTriggerCondition]): Specification of which price component
                should be used when determining if an Order should be triggered and filled. This allows Orders to be triggered
                based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy) price
                depending on the desired behaviour. Orders are always filled using their default price component.
                This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
                condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
                transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
                is set to the default value when indicating the distance from an Order's trigger price, and will always provide
                the default trigger condition when creating or modifying an Order.
                A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
                value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
                a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
            guaranteed (Optional[bool]): Flag indicating that the Stop Loss Order is guaranteed. The default value
                depends on the GuaranteedStopLossOrderMode of the account, if it is REQUIRED, the default will be true, for
                DISABLED or ENABLED the default is false.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4."""

    type: Optional[StopLossOrderRequestType]
    trade_id: Optional[str]
    client_trade_id: Optional[str]
    price: Optional[str]
    distance: Optional[str]
    time_in_force: Optional[StopLossOrderRequestTimeInForce]
    gtd_time: Optional[str]
    trigger_condition: Optional[StopLossOrderRequestTriggerCondition]
    guaranteed: Optional[bool]
    client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        _type = d.pop("type", None)
        type: Optional[StopLossOrderRequestType]
        if _type is None:
            type = None
        else:
            type = check_stop_loss_order_request_type(_type)
        trade_id = d.pop("tradeID", None)
        client_trade_id = d.pop("clientTradeID", None)
        price = d.pop("price", None)
        distance = d.pop("distance", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[StopLossOrderRequestTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_stop_loss_order_request_time_in_force(_time_in_force)
        gtd_time = d.pop("gtdTime", None)
        _trigger_condition = d.pop("triggerCondition", None)
        trigger_condition: Optional[StopLossOrderRequestTriggerCondition]
        if _trigger_condition is None:
            trigger_condition = None
        else:
            trigger_condition = check_stop_loss_order_request_trigger_condition(
                _trigger_condition
            )
        guaranteed = d.pop("guaranteed", None)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        stop_loss_order_request = cls(
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            price=price,
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            guaranteed=guaranteed,
            client_extensions=client_extensions,
        )
        stop_loss_order_request.additional_properties = d
        return stop_loss_order_request

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="CloseTradeBody")


@dataclasses.dataclass
class CloseTradeBody:
    """Attributes:
    units (Optional[str]): Indication of how much of the Trade to close. Either the string "ALL" (indicating
        that all of the Trade should be closed), or a DecimalNumber representing the number of units of the open Trade
        to Close using a TradeClose MarketOrder. The units specified must always be positive, and the magnitude of the
        value cannot exceed the magnitude of the Trade's open units."""

    units: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        units = d.pop("units", None)
        close_trade_body = cls(units=units)
        close_trade_body.additional_properties = d
        return close_trade_body

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TradeOpen")


@dataclasses.dataclass
class TradeOpen:
    """A TradeOpen object represents a Trade for an instrument that was opened in an Account. It is found embedded in
    Transactions that affect the position of an instrument in the Account, specifically the OrderFill Transaction.

        Attributes:
            trade_id (Optional[str]): The ID of the Trade that was opened
            units (Optional[str]): The number of units opened by the Trade
            price (Optional[str]): The average price that the units were opened at.
            guaranteed_execution_fee (Optional[str]): This is the fee charged for opening the trade if it has a
                guaranteed Stop Loss Order attached to it.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            half_spread_cost (Optional[str]): The half spread cost for the trade open. This can be a positive or
                negative value and is represented in the home currency of the Account.
            initial_margin_required (Optional[str]): The margin required at the time the Trade was created. Note, this
                is the 'pure' margin required, it is not the 'effective' margin used that factors in the trade risk if a GSLO is
                attached to the trade."""

    trade_id: Optional[str]
    units: Optional[str]
    price: Optional[str]
    guaranteed_execution_fee: Optional[str]
    client_extensions: Optional["ClientExtensions"]
    half_spread_cost: Optional[str]
    initial_margin_required: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        trade_id = d.pop("tradeID", None)
        units = d.pop("units", None)
        price = d.pop("price", None)
        guaranteed_execution_fee = d.pop("guaranteedExecutionFee", None)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        half_spread_cost = d.pop("halfSpreadCost", None)
        initial_margin_required = d.pop("initialMarginRequired", None)
        trade_open = cls(
            trade_id=trade_id,
            units=units,
            price=price,
            guaranteed_execution_fee=guaranteed_execution_fee,
            client_extensions=client_extensions,
            half_spread_cost=half_spread_cost,
            initial_margin_required=initial_margin_required,
        )
        trade_open.additional_properties = d
        return trade_open

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="SetOrderClientExtensionsBody")


@dataclasses.dataclass
class SetOrderClientExtensionsBody:
    """Attributes:
    client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
        clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
        your account is associated with MT4.
    trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
        clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
        your account is associated with MT4."""

    client_extensions: Optional["ClientExtensions"]
    trade_client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        set_order_client_extensions_body = cls(
            client_extensions=client_extensions,
            trade_client_extensions=trade_client_extensions,
        )
        set_order_client_extensions_body.additional_properties = d
        return set_order_client_extensions_body

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

MarketIfTouchedOrderRejectTransactionReason = Literal["CLIENT_ORDER", "REPLACEMENT"]
MARKET_IF_TOUCHED_ORDER_REJECT_TRANSACTION_REASON_VALUES: Set[
    MarketIfTouchedOrderRejectTransactionReason
] = {"CLIENT_ORDER", "REPLACEMENT"}


def check_market_if_touched_order_reject_transaction_reason(
    value: str,
) -> MarketIfTouchedOrderRejectTransactionReason:
    if value in MARKET_IF_TOUCHED_ORDER_REJECT_TRANSACTION_REASON_VALUES:
        return cast(MarketIfTouchedOrderRejectTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_IF_TOUCHED_ORDER_REJECT_TRANSACTION_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.stop_loss_order_transaction_reason import (
    check_stop_loss_order_transaction_reason,
)
from ..models.stop_loss_order_transaction_reason import StopLossOrderTransactionReason
from ..models.stop_loss_order_transaction_time_in_force import (
    check_stop_loss_order_transaction_time_in_force,
)
from ..models.stop_loss_order_transaction_time_in_force import (
    StopLossOrderTransactionTimeInForce,
)
from ..models.stop_loss_order_transaction_trigger_condition import (
    check_stop_loss_order_transaction_trigger_condition,
)
from ..models.stop_loss_order_transaction_trigger_condition import (
    StopLossOrderTransactionTriggerCondition,
)
from ..models.stop_loss_order_transaction_type import (
    check_stop_loss_order_transaction_type,
)
from ..models.stop_loss_order_transaction_type import StopLossOrderTransactionType
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="StopLossOrderTransaction")


@_attrs_define
class StopLossOrderTransaction:
    """A StopLossOrderTransaction represents the creation of a StopLoss Order in the user's Account.

    Attributes:
        id (Union[Unset, str]): The Transaction's Identifier.
        time (Union[Unset, str]): The date/time when the Transaction was created.
        user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
        batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
        type (Union[Unset, StopLossOrderTransactionType]): The Type of the Transaction. Always set to "STOP_LOSS_ORDER"
            in a StopLossOrderTransaction.
        trade_id (Union[Unset, str]): The ID of the Trade to close when the price threshold is breached.
        client_trade_id (Union[Unset, str]): The client ID of the Trade to be closed when the price threshold is
            breached.
        price (Union[Unset, str]): The price threshold specified for the Stop Loss Order. If the guaranteed flag is
            false, the associated Trade will be closed by a market price that is equal to or worse than this threshold. If
            the flag is true the associated Trade will be closed at this price.
        distance (Union[Unset, str]): Specifies the distance (in price units) from the Account's current price to use as
            the Stop Loss Order price. If the Trade is short the Instrument's bid price is used, and for long Trades the ask
            is used.
        time_in_force (Union[Unset, StopLossOrderTransactionTimeInForce]): The time-in-force requested for the StopLoss
            Order. Restricted to "GTC", "GFD" and "GTD" for StopLoss Orders.
        gtd_time (Union[Unset, str]): The date/time when the StopLoss Order will be cancelled if its timeInForce is
            "GTD".
        trigger_condition (Union[Unset, StopLossOrderTransactionTriggerCondition]): Specification of which price
            component should be used when determining if an Order should be triggered and filled. This allows Orders to be
            triggered based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy)
            price depending on the desired behaviour. Orders are always filled using their default price component.
            This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
            condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
            transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
            is set to the default value when indicating the distance from an Order's trigger price, and will always provide
            the default trigger condition when creating or modifying an Order.
            A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
            value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
            a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
        guaranteed (Union[Unset, bool]): Flag indicating that the Stop Loss Order is guaranteed. The default value
            depends on the GuaranteedStopLossOrderMode of the account, if it is REQUIRED, the default will be true, for
            DISABLED or ENABLED the default is false.
        guaranteed_execution_premium (Union[Unset, str]): The fee that will be charged if the Stop Loss Order is
            guaranteed and the Order is filled at the guaranteed price. The value is determined at Order creation time. It
            is in price units and is charged for each unit of the Trade.
        reason (Union[Unset, StopLossOrderTransactionReason]): The reason that the Stop Loss Order was initiated
        client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        order_fill_transaction_id (Union[Unset, str]): The ID of the OrderFill Transaction that caused this Order to be
            created (only provided if this Order was created automatically when another Order was filled).
        replaces_order_id (Union[Unset, str]): The ID of the Order that this Order replaces (only provided if this Order
            replaces an existing Order).
        cancelling_transaction_id (Union[Unset, str]): The ID of the Transaction that cancels the replaced Order (only
            provided if this Order replaces an existing Order).
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, StopLossOrderTransactionType] = UNSET
    trade_id: Union[Unset, str] = UNSET
    client_trade_id: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    distance: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, StopLossOrderTransactionTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    trigger_condition: Union[Unset, StopLossOrderTransactionTriggerCondition] = UNSET
    guaranteed: Union[Unset, bool] = UNSET
    guaranteed_execution_premium: Union[Unset, str] = UNSET
    reason: Union[Unset, StopLossOrderTransactionReason] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    order_fill_transaction_id: Union[Unset, str] = UNSET
    replaces_order_id: Union[Unset, str] = UNSET
    cancelling_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        trade_id = self.trade_id

        client_trade_id = self.client_trade_id

        price = self.price

        distance = self.distance

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        trigger_condition: Union[Unset, str] = UNSET
        if not isinstance(self.trigger_condition, Unset):
            trigger_condition = self.trigger_condition

        guaranteed = self.guaranteed

        guaranteed_execution_premium = self.guaranteed_execution_premium

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        order_fill_transaction_id = self.order_fill_transaction_id

        replaces_order_id = self.replaces_order_id

        cancelling_transaction_id = self.cancelling_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type
        if trade_id is not UNSET:
            field_dict["tradeID"] = trade_id
        if client_trade_id is not UNSET:
            field_dict["clientTradeID"] = client_trade_id
        if price is not UNSET:
            field_dict["price"] = price
        if distance is not UNSET:
            field_dict["distance"] = distance
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if trigger_condition is not UNSET:
            field_dict["triggerCondition"] = trigger_condition
        if guaranteed is not UNSET:
            field_dict["guaranteed"] = guaranteed
        if guaranteed_execution_premium is not UNSET:
            field_dict["guaranteedExecutionPremium"] = guaranteed_execution_premium
        if reason is not UNSET:
            field_dict["reason"] = reason
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if order_fill_transaction_id is not UNSET:
            field_dict["orderFillTransactionID"] = order_fill_transaction_id
        if replaces_order_id is not UNSET:
            field_dict["replacesOrderID"] = replaces_order_id
        if cancelling_transaction_id is not UNSET:
            field_dict["cancellingTransactionID"] = cancelling_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, StopLossOrderTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_stop_loss_order_transaction_type(_type)

        trade_id = d.pop("tradeID", UNSET)

        client_trade_id = d.pop("clientTradeID", UNSET)

        price = d.pop("price", UNSET)

        distance = d.pop("distance", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, StopLossOrderTransactionTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_stop_loss_order_transaction_time_in_force(
                _time_in_force
            )

        gtd_time = d.pop("gtdTime", UNSET)

        _trigger_condition = d.pop("triggerCondition", UNSET)
        trigger_condition: Union[Unset, StopLossOrderTransactionTriggerCondition]
        if isinstance(_trigger_condition, Unset):
            trigger_condition = UNSET
        else:
            trigger_condition = check_stop_loss_order_transaction_trigger_condition(
                _trigger_condition
            )

        guaranteed = d.pop("guaranteed", UNSET)

        guaranteed_execution_premium = d.pop("guaranteedExecutionPremium", UNSET)

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, StopLossOrderTransactionReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = check_stop_loss_order_transaction_reason(_reason)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        order_fill_transaction_id = d.pop("orderFillTransactionID", UNSET)

        replaces_order_id = d.pop("replacesOrderID", UNSET)

        cancelling_transaction_id = d.pop("cancellingTransactionID", UNSET)

        stop_loss_order_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            price=price,
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            guaranteed=guaranteed,
            guaranteed_execution_premium=guaranteed_execution_premium,
            reason=reason,
            client_extensions=client_extensions,
            order_fill_transaction_id=order_fill_transaction_id,
            replaces_order_id=replaces_order_id,
            cancelling_transaction_id=cancelling_transaction_id,
        )

        stop_loss_order_transaction.additional_properties = d
        return stop_loss_order_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .trade import Trade
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="ListOpenTradesResponse200")


@dataclasses.dataclass
class ListOpenTradesResponse200:
    """Attributes:
    trades (Optional[List['Trade']]): The Account's list of open Trades
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    trades: Optional[List["Trade"]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .trade import Trade

        d = src_dict.copy()
        trades = []
        _trades = d.pop("trades", None)
        for trades_item_data in _trades or []:
            trades_item = Trade.from_dict(trades_item_data)
            trades.append(trades_item)
        last_transaction_id = d.pop("lastTransactionID", None)
        list_open_trades_response_200 = cls(
            trades=trades, last_transaction_id=last_transaction_id
        )
        list_open_trades_response_200.additional_properties = d
        return list_open_trades_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.get_account_instruments_response_200 import (
    GetAccountInstrumentsResponse200,
)
from ...types import Unset


def _get_kwargs(
    account_id: str,
    *,
    instruments: Union[Unset, List[str]] = UNSET,
    authorization: str,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["Authorization"] = authorization

    params: Dict[str, Any] = {}

    json_instruments: Union[Unset, List[str]] = UNSET
    if not isinstance(instruments, Unset):
        json_instruments = instruments

    params["instruments"] = json_instruments

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/accounts/{account_id}/instruments".format(
            account_id=account_id,
        ),
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, GetAccountInstrumentsResponse200]]:
    if response.status_code == 200:
        response_200 = GetAccountInstrumentsResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 405:
        response_405 = cast(Any, None)
        return response_405
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, GetAccountInstrumentsResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: Union[Unset, List[str]] = UNSET,
    authorization: str,
) -> Response[Union[Any, GetAccountInstrumentsResponse200]]:
    """Account Instruments

     Get the list of tradeable instruments for the given Account. The list of tradeable instruments is
    dependent on the regulatory division that the Account is located in, thus should be the same for all
    Accounts owned by a single user.

    Args:
        account_id (str):
        instruments (Union[Unset, List[str]]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, GetAccountInstrumentsResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        instruments=instruments,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: Union[Unset, List[str]] = UNSET,
    authorization: str,
) -> Optional[Union[Any, GetAccountInstrumentsResponse200]]:
    """Account Instruments

     Get the list of tradeable instruments for the given Account. The list of tradeable instruments is
    dependent on the regulatory division that the Account is located in, thus should be the same for all
    Accounts owned by a single user.

    Args:
        account_id (str):
        instruments (Union[Unset, List[str]]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, GetAccountInstrumentsResponse200]
    """

    return sync_detailed(
        account_id=account_id,
        client=client,
        instruments=instruments,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: Union[Unset, List[str]] = UNSET,
    authorization: str,
) -> Response[Union[Any, GetAccountInstrumentsResponse200]]:
    """Account Instruments

     Get the list of tradeable instruments for the given Account. The list of tradeable instruments is
    dependent on the regulatory division that the Account is located in, thus should be the same for all
    Accounts owned by a single user.

    Args:
        account_id (str):
        instruments (Union[Unset, List[str]]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, GetAccountInstrumentsResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        instruments=instruments,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    instruments: Union[Unset, List[str]] = UNSET,
    authorization: str,
) -> Optional[Union[Any, GetAccountInstrumentsResponse200]]:
    """Account Instruments

     Get the list of tradeable instruments for the given Account. The list of tradeable instruments is
    dependent on the regulatory division that the Account is located in, thus should be the same for all
    Accounts owned by a single user.

    Args:
        account_id (str):
        instruments (Union[Unset, List[str]]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, GetAccountInstrumentsResponse200]
    """

    return (
        await asyncio_detailed(
            account_id=account_id,
            client=client,
            instruments=instruments,
            authorization=authorization,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.get_instrument_price_response_200 import GetInstrumentPriceResponse200
from ...types import Unset


def _get_kwargs(
    instrument: str,
    *,
    time: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["Authorization"] = authorization

    if not isinstance(accept_datetime_format, Unset):
        headers["Accept-Datetime-Format"] = accept_datetime_format

    params: Dict[str, Any] = {}

    params["time"] = time

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/instruments/{instrument}/price".format(
            instrument=instrument,
        ),
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, GetInstrumentPriceResponse200]]:
    if response.status_code == 200:
        response_200 = GetInstrumentPriceResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 405:
        response_405 = cast(Any, None)
        return response_405
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, GetInstrumentPriceResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    instrument: str,
    *,
    client: Union[AuthenticatedClient, Client],
    time: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, GetInstrumentPriceResponse200]]:
    """Price

     Fetch a price for an instrument. Accounts are not associated in any way with this endpoint.

    Args:
        instrument (str):
        time (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, GetInstrumentPriceResponse200]]
    """

    kwargs = _get_kwargs(
        instrument=instrument,
        time=time,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    instrument: str,
    *,
    client: Union[AuthenticatedClient, Client],
    time: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, GetInstrumentPriceResponse200]]:
    """Price

     Fetch a price for an instrument. Accounts are not associated in any way with this endpoint.

    Args:
        instrument (str):
        time (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, GetInstrumentPriceResponse200]
    """

    return sync_detailed(
        instrument=instrument,
        client=client,
        time=time,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    ).parsed


async def asyncio_detailed(
    instrument: str,
    *,
    client: Union[AuthenticatedClient, Client],
    time: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Response[Union[Any, GetInstrumentPriceResponse200]]:
    """Price

     Fetch a price for an instrument. Accounts are not associated in any way with this endpoint.

    Args:
        instrument (str):
        time (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, GetInstrumentPriceResponse200]]
    """

    kwargs = _get_kwargs(
        instrument=instrument,
        time=time,
        authorization=authorization,
        accept_datetime_format=accept_datetime_format,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    instrument: str,
    *,
    client: Union[AuthenticatedClient, Client],
    time: Union[Unset, str] = UNSET,
    authorization: str,
    accept_datetime_format: Union[Unset, str] = UNSET,
) -> Optional[Union[Any, GetInstrumentPriceResponse200]]:
    """Price

     Fetch a price for an instrument. Accounts are not associated in any way with this endpoint.

    Args:
        instrument (str):
        time (Union[Unset, str]):
        authorization (str):
        accept_datetime_format (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, GetInstrumentPriceResponse200]
    """

    return (
        await asyncio_detailed(
            instrument=instrument,
            client=client,
            time=time,
            authorization=authorization,
            accept_datetime_format=accept_datetime_format,
        )
    ).parsed



from __future__ import annotations
from typing import Literal, Set, cast

TradeState = Literal["CLOSE_WHEN_TRADEABLE", "CLOSED", "OPEN"]
TRADE_STATE_VALUES: Set[TradeState] = {"CLOSE_WHEN_TRADEABLE", "CLOSED", "OPEN"}


def check_trade_state(value: str) -> TradeState:
    if value in TRADE_STATE_VALUES:
        return cast(TradeState, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRADE_STATE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.take_profit_order_request_time_in_force import (
    check_take_profit_order_request_time_in_force,
)
from ..models.take_profit_order_request_time_in_force import (
    TakeProfitOrderRequestTimeInForce,
)
from ..models.take_profit_order_request_trigger_condition import (
    check_take_profit_order_request_trigger_condition,
)
from ..models.take_profit_order_request_trigger_condition import (
    TakeProfitOrderRequestTriggerCondition,
)
from ..models.take_profit_order_request_type import check_take_profit_order_request_type
from ..models.take_profit_order_request_type import TakeProfitOrderRequestType
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="TakeProfitOrderRequest")


@_attrs_define
class TakeProfitOrderRequest:
    """A TakeProfitOrderRequest specifies the parameters that may be set when creating a Take Profit Order. Only one of the
    price and distance fields may be specified.

        Attributes:
            type (Union[Unset, TakeProfitOrderRequestType]): The type of the Order to Create. Must be set to "TAKE_PROFIT"
                when creating a Take Profit Order.
            trade_id (Union[Unset, str]): The ID of the Trade to close when the price threshold is breached.
            client_trade_id (Union[Unset, str]): The client ID of the Trade to be closed when the price threshold is
                breached.
            price (Union[Unset, str]): The price threshold specified for the TakeProfit Order. The associated Trade will be
                closed by a market price that is equal to or better than this threshold.
            time_in_force (Union[Unset, TakeProfitOrderRequestTimeInForce]): The time-in-force requested for the TakeProfit
                Order. Restricted to "GTC", "GFD" and "GTD" for TakeProfit Orders.
            gtd_time (Union[Unset, str]): The date/time when the TakeProfit Order will be cancelled if its timeInForce is
                "GTD".
            trigger_condition (Union[Unset, TakeProfitOrderRequestTriggerCondition]): Specification of which price component
                should be used when determining if an Order should be triggered and filled. This allows Orders to be triggered
                based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy) price
                depending on the desired behaviour. Orders are always filled using their default price component.
                This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
                condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
                transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
                is set to the default value when indicating the distance from an Order's trigger price, and will always provide
                the default trigger condition when creating or modifying an Order.
                A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
                value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
                a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
    """

    type: Union[Unset, TakeProfitOrderRequestType] = UNSET
    trade_id: Union[Unset, str] = UNSET
    client_trade_id: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, TakeProfitOrderRequestTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    trigger_condition: Union[Unset, TakeProfitOrderRequestTriggerCondition] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        trade_id = self.trade_id

        client_trade_id = self.client_trade_id

        price = self.price

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        trigger_condition: Union[Unset, str] = UNSET
        if not isinstance(self.trigger_condition, Unset):
            trigger_condition = self.trigger_condition

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if trade_id is not UNSET:
            field_dict["tradeID"] = trade_id
        if client_trade_id is not UNSET:
            field_dict["clientTradeID"] = client_trade_id
        if price is not UNSET:
            field_dict["price"] = price
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if trigger_condition is not UNSET:
            field_dict["triggerCondition"] = trigger_condition
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, TakeProfitOrderRequestType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_take_profit_order_request_type(_type)

        trade_id = d.pop("tradeID", UNSET)

        client_trade_id = d.pop("clientTradeID", UNSET)

        price = d.pop("price", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, TakeProfitOrderRequestTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_take_profit_order_request_time_in_force(
                _time_in_force
            )

        gtd_time = d.pop("gtdTime", UNSET)

        _trigger_condition = d.pop("triggerCondition", UNSET)
        trigger_condition: Union[Unset, TakeProfitOrderRequestTriggerCondition]
        if isinstance(_trigger_condition, Unset):
            trigger_condition = UNSET
        else:
            trigger_condition = check_take_profit_order_request_trigger_condition(
                _trigger_condition
            )

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        take_profit_order_request = cls(
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            price=price,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            client_extensions=client_extensions,
        )

        take_profit_order_request.additional_properties = d
        return take_profit_order_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .candlestick_data import CandlestickData
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="Candlestick")


@dataclasses.dataclass
class Candlestick:
    """The Candlestick representation

    Attributes:
        time (Optional[str]): The start time of the candlestick
        bid (Optional[CandlestickData]): The price data (open, high, low, close) for the Candlestick representation.
        ask (Optional[CandlestickData]): The price data (open, high, low, close) for the Candlestick representation.
        mid (Optional[CandlestickData]): The price data (open, high, low, close) for the Candlestick representation.
        volume (Optional[int]): The number of prices created during the time-range represented by the candlestick.
        complete (Optional[bool]): A flag indicating if the candlestick is complete. A complete candlestick is one
            whose ending time is not in the future."""

    time: Optional[str]
    bid: Optional["CandlestickData"]
    ask: Optional["CandlestickData"]
    mid: Optional["CandlestickData"]
    volume: Optional[int]
    complete: Optional[bool]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .candlestick_data import CandlestickData

        d = src_dict.copy()
        time = d.pop("time", None)
        _bid = d.pop("bid", None)
        bid: Optional[CandlestickData]
        if _bid is None:
            bid = None
        else:
            bid = CandlestickData.from_dict(_bid)
        _ask = d.pop("ask", None)
        ask: Optional[CandlestickData]
        if _ask is None:
            ask = None
        else:
            ask = CandlestickData.from_dict(_ask)
        _mid = d.pop("mid", None)
        mid: Optional[CandlestickData]
        if _mid is None:
            mid = None
        else:
            mid = CandlestickData.from_dict(_mid)
        volume = d.pop("volume", None)
        complete = d.pop("complete", None)
        candlestick = cls(
            time=time, bid=bid, ask=ask, mid=mid, volume=volume, complete=complete
        )
        candlestick.additional_properties = d
        return candlestick

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

LimitOrderType = Literal[
    "FIXED_PRICE",
    "LIMIT",
    "MARKET",
    "MARKET_IF_TOUCHED",
    "STOP",
    "STOP_LOSS",
    "TAKE_PROFIT",
    "TRAILING_STOP_LOSS",
]

LIMIT_ORDER_TYPE_VALUES: Set[LimitOrderType] = {
    "FIXED_PRICE",
    "LIMIT",
    "MARKET",
    "MARKET_IF_TOUCHED",
    "STOP",
    "STOP_LOSS",
    "TAKE_PROFIT",
    "TRAILING_STOP_LOSS",
}


def check_limit_order_type(value: str) -> LimitOrderType:
    if value in LIMIT_ORDER_TYPE_VALUES:
        return cast(LimitOrderType, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_TYPE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="DynamicOrderState")


@_attrs_define
class DynamicOrderState:
    """The dynamic state of an Order. This is only relevant to TrailingStopLoss Orders, as no other Order type has dynamic
    state.

        Attributes:
            id (Union[Unset, str]): The Order's ID.
            trailing_stop_value (Union[Unset, str]): The Order's calculated trailing stop value.
            trigger_distance (Union[Unset, str]): The distance between the Trailing Stop Loss Order's trailingStopValue and
                the current Market Price. This represents the distance (in price units) of the Order from a triggering price. If
                the distance could not be determined, this value will not be set.
            is_trigger_distance_exact (Union[Unset, bool]): True if an exact trigger distance could be calculated. If false,
                it means the provided trigger distance is a best estimate. If the distance could not be determined, this value
                will not be set.
    """

    id: Union[Unset, str] = UNSET
    trailing_stop_value: Union[Unset, str] = UNSET
    trigger_distance: Union[Unset, str] = UNSET
    is_trigger_distance_exact: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        trailing_stop_value = self.trailing_stop_value

        trigger_distance = self.trigger_distance

        is_trigger_distance_exact = self.is_trigger_distance_exact

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if trailing_stop_value is not UNSET:
            field_dict["trailingStopValue"] = trailing_stop_value
        if trigger_distance is not UNSET:
            field_dict["triggerDistance"] = trigger_distance
        if is_trigger_distance_exact is not UNSET:
            field_dict["isTriggerDistanceExact"] = is_trigger_distance_exact

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        trailing_stop_value = d.pop("trailingStopValue", UNSET)

        trigger_distance = d.pop("triggerDistance", UNSET)

        is_trigger_distance_exact = d.pop("isTriggerDistanceExact", UNSET)

        dynamic_order_state = cls(
            id=id,
            trailing_stop_value=trailing_stop_value,
            trigger_distance=trigger_distance,
            is_trigger_distance_exact=is_trigger_distance_exact,
        )

        dynamic_order_state.additional_properties = d
        return dynamic_order_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

OrderState = Literal["CANCELLED", "FILLED", "PENDING", "TRIGGERED"]
ORDER_STATE_VALUES: Set[OrderState] = {"CANCELLED", "FILLED", "PENDING", "TRIGGERED"}


def check_order_state(value: str) -> OrderState:
    if value in ORDER_STATE_VALUES:
        return cast(OrderState, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ORDER_STATE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .order_state import OrderState
from .order_state import check_order_state
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="Order")


@dataclasses.dataclass
class Order:
    """The base Order definition specifies the properties that are common to all Orders.

    Attributes:
        id (Optional[str]): The Order's identifier, unique within the Order's Account.
        create_time (Optional[str]): The time when the Order was created.
        state (Optional[OrderState]): The current state of the Order.
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4."""

    id: Optional[str]
    create_time: Optional[str]
    state: Optional[OrderState]
    client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", None)
        create_time = d.pop("createTime", None)
        _state = d.pop("state", None)
        state: Optional[OrderState]
        if _state is None:
            state = None
        else:
            state = check_order_state(_state)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
        )
        order.additional_properties = d
        return order

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.get_instrument_candles_response_200_granularity import (
    check_get_instrument_candles_response_200_granularity,
)
from ..models.get_instrument_candles_response_200_granularity import (
    GetInstrumentCandlesResponse200Granularity,
)
from typing import Union

if TYPE_CHECKING:
    from ..models.candlestick import Candlestick


T = TypeVar("T", bound="GetInstrumentCandlesResponse200")


@_attrs_define
class GetInstrumentCandlesResponse200:
    """
    Attributes:
        instrument (Union[Unset, str]): The instrument whose Prices are represented by the candlesticks.
        granularity (Union[Unset, GetInstrumentCandlesResponse200Granularity]): The granularity of the candlesticks
            provided.
        candles (Union[Unset, List['Candlestick']]): The list of candlesticks that satisfy the request.
    """

    instrument: Union[Unset, str] = UNSET
    granularity: Union[Unset, GetInstrumentCandlesResponse200Granularity] = UNSET
    candles: Union[Unset, List["Candlestick"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        instrument = self.instrument

        granularity: Union[Unset, str] = UNSET
        if not isinstance(self.granularity, Unset):
            granularity = self.granularity

        candles: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.candles, Unset):
            candles = []
            for candles_item_data in self.candles:
                candles_item = candles_item_data.to_dict()
                candles.append(candles_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if granularity is not UNSET:
            field_dict["granularity"] = granularity
        if candles is not UNSET:
            field_dict["candles"] = candles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.candlestick import Candlestick

        d = src_dict.copy()
        instrument = d.pop("instrument", UNSET)

        _granularity = d.pop("granularity", UNSET)
        granularity: Union[Unset, GetInstrumentCandlesResponse200Granularity]
        if isinstance(_granularity, Unset):
            granularity = UNSET
        else:
            granularity = check_get_instrument_candles_response_200_granularity(
                _granularity
            )

        candles = []
        _candles = d.pop("candles", UNSET)
        for candles_item_data in _candles or []:
            candles_item = Candlestick.from_dict(candles_item_data)

            candles.append(candles_item)

        get_instrument_candles_response_200 = cls(
            instrument=instrument,
            granularity=granularity,
            candles=candles,
        )

        get_instrument_candles_response_200.additional_properties = d
        return get_instrument_candles_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

MarketOrderMarginCloseoutReason = Literal[
    "MARGIN_CHECK_VIOLATION",
    "REGULATORY_MARGIN_CALL_VIOLATION",
    "REGULATORY_MARGIN_CHECK_VIOLATION",
]
MARKET_ORDER_MARGIN_CLOSEOUT_REASON_VALUES: Set[MarketOrderMarginCloseoutReason] = {
    "MARGIN_CHECK_VIOLATION",
    "REGULATORY_MARGIN_CALL_VIOLATION",
    "REGULATORY_MARGIN_CHECK_VIOLATION",
}


def check_market_order_margin_closeout_reason(
    value: str,
) -> MarketOrderMarginCloseoutReason:
    if value in MARKET_ORDER_MARGIN_CLOSEOUT_REASON_VALUES:
        return cast(MarketOrderMarginCloseoutReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_ORDER_MARGIN_CLOSEOUT_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

MarketOrderRejectTransactionTimeInForce = Literal["FOK", "GFD", "GTC", "GTD", "IOC"]
MARKET_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES: Set[
    MarketOrderRejectTransactionTimeInForce
] = {"FOK", "GFD", "GTC", "GTD", "IOC"}


def check_market_order_reject_transaction_time_in_force(
    value: str,
) -> MarketOrderRejectTransactionTimeInForce:
    if value in MARKET_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES:
        return cast(MarketOrderRejectTransactionTimeInForce, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {MARKET_ORDER_REJECT_TRANSACTION_TIME_IN_FORCE_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.guaranteed_stop_loss_order_level_restriction import (
        GuaranteedStopLossOrderLevelRestriction,
    )


T = TypeVar("T", bound="GuaranteedStopLossOrderEntryData")


@_attrs_define
class GuaranteedStopLossOrderEntryData:
    """Details required by clients creating a Guaranteed Stop Loss Order

    Attributes:
        minimum_distance (Union[Unset, str]): The minimum distance allowed between the Trade's fill price and the
            configured price for guaranteed Stop Loss Orders created for this instrument. Specified in price units.
        premium (Union[Unset, str]): The amount that is charged to the account if a guaranteed Stop Loss Order is
            triggered and filled. The value is in price units and is charged for each unit of the Trade.
        level_restriction (Union[Unset, GuaranteedStopLossOrderLevelRestriction]): A
            GuaranteedStopLossOrderLevelRestriction represents the total position size that can exist within a given price
            window for Trades with guaranteed Stop Loss Orders attached for a specific Instrument.
    """

    minimum_distance: Union[Unset, str] = UNSET
    premium: Union[Unset, str] = UNSET
    level_restriction: Union[Unset, "GuaranteedStopLossOrderLevelRestriction"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        minimum_distance = self.minimum_distance

        premium = self.premium

        level_restriction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.level_restriction, Unset):
            level_restriction = self.level_restriction.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if minimum_distance is not UNSET:
            field_dict["minimumDistance"] = minimum_distance
        if premium is not UNSET:
            field_dict["premium"] = premium
        if level_restriction is not UNSET:
            field_dict["levelRestriction"] = level_restriction

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.guaranteed_stop_loss_order_level_restriction import (
            GuaranteedStopLossOrderLevelRestriction,
        )

        d = src_dict.copy()
        minimum_distance = d.pop("minimumDistance", UNSET)

        premium = d.pop("premium", UNSET)

        _level_restriction = d.pop("levelRestriction", UNSET)
        level_restriction: Union[Unset, GuaranteedStopLossOrderLevelRestriction]
        if isinstance(_level_restriction, Unset):
            level_restriction = UNSET
        else:
            level_restriction = GuaranteedStopLossOrderLevelRestriction.from_dict(
                _level_restriction
            )

        guaranteed_stop_loss_order_entry_data = cls(
            minimum_distance=minimum_distance,
            premium=premium,
            level_restriction=level_restriction,
        )

        guaranteed_stop_loss_order_entry_data.additional_properties = d
        return guaranteed_stop_loss_order_entry_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.client_price import ClientPrice
    from ..models.pricing_heartbeat import PricingHeartbeat


T = TypeVar("T", bound="StreamPricingResponse200")


@_attrs_define
class StreamPricingResponse200:
    """The response body for the Pricing Stream uses chunked transfer encoding.  Each chunk contains Price and/or
    PricingHeartbeat objects encoded as JSON.  Each JSON object is serialized into a single line of text, and multiple
    objects found in the same chunk are separated by newlines.
    Heartbeats are sent every 5 seconds.

        Attributes:
            price (Union[Unset, ClientPrice]): The specification of an Account-specific Price.
            heartbeat (Union[Unset, PricingHeartbeat]): A PricingHeartbeat object is injected into the Pricing stream to
                ensure that the HTTP connection remains active.
    """

    price: Union[Unset, "ClientPrice"] = UNSET
    heartbeat: Union[Unset, "PricingHeartbeat"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        price: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.price, Unset):
            price = self.price.to_dict()

        heartbeat: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.heartbeat, Unset):
            heartbeat = self.heartbeat.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if price is not UNSET:
            field_dict["price"] = price
        if heartbeat is not UNSET:
            field_dict["heartbeat"] = heartbeat

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_price import ClientPrice
        from ..models.pricing_heartbeat import PricingHeartbeat

        d = src_dict.copy()
        _price = d.pop("price", UNSET)
        price: Union[Unset, ClientPrice]
        if isinstance(_price, Unset):
            price = UNSET
        else:
            price = ClientPrice.from_dict(_price)

        _heartbeat = d.pop("heartbeat", UNSET)
        heartbeat: Union[Unset, PricingHeartbeat]
        if isinstance(_heartbeat, Unset):
            heartbeat = UNSET
        else:
            heartbeat = PricingHeartbeat.from_dict(_heartbeat)

        stream_pricing_response_200 = cls(
            price=price,
            heartbeat=heartbeat,
        )

        stream_pricing_response_200.additional_properties = d
        return stream_pricing_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="HomeConversions")


@dataclasses.dataclass
class HomeConversions:
    """HomeConversions represents the factors to use to convert quantities of a given currency into the Account's home
    currency. The conversion factor depends on the scenario the conversion is required for.

        Attributes:
            currency (Optional[str]): The currency to be converted into the home currency.
            account_gain (Optional[str]): The factor used to convert any gains for an Account in the specified currency
                into the Account's home currency. This would include positive realized P/L and positive financing amounts.
                Conversion is performed by multiplying the positive P/L by the conversion factor.
            account_loss (Optional[str]): The string representation of a decimal number.
            position_value (Optional[str]): The factor used to convert a Position or Trade Value in the specified
                currency into the Account's home currency. Conversion is performed by multiplying the Position or Trade Value by
                the conversion factor."""

    currency: Optional[str]
    account_gain: Optional[str]
    account_loss: Optional[str]
    position_value: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        currency = d.pop("currency", None)
        account_gain = d.pop("accountGain", None)
        account_loss = d.pop("accountLoss", None)
        position_value = d.pop("positionValue", None)
        home_conversions = cls(
            currency=currency,
            account_gain=account_gain,
            account_loss=account_loss,
            position_value=position_value,
        )
        home_conversions.additional_properties = d
        return home_conversions

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GuaranteedStopLossOrderLevelRestriction")


@dataclasses.dataclass
class GuaranteedStopLossOrderLevelRestriction:
    """A GuaranteedStopLossOrderLevelRestriction represents the total position size that can exist within a given price
    window for Trades with guaranteed Stop Loss Orders attached for a specific Instrument.

        Attributes:
            volume (Optional[str]): Applies to Trades with a guaranteed Stop Loss Order attached for the specified
                Instrument. This is the total allowed Trade volume that can exist within the priceRange based on the trigger
                prices of the guaranteed Stop Loss Orders.
            price_range (Optional[str]): The price range the volume applies to. This value is in price units."""

    volume: Optional[str]
    price_range: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        volume = d.pop("volume", None)
        price_range = d.pop("priceRange", None)
        guaranteed_stop_loss_order_level_restriction = cls(
            volume=volume, price_range=price_range
        )
        guaranteed_stop_loss_order_level_restriction.additional_properties = d
        return guaranteed_stop_loss_order_level_restriction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

StopOrderRejectTransactionRejectReason = Literal[
    "ACCOUNT_CONFIGURATION_LOCKED",
    "ACCOUNT_DEPOSIT_LOCKED",
    "ACCOUNT_LOCKED",
    "ACCOUNT_NOT_ACTIVE",
    "ACCOUNT_ORDER_CANCEL_LOCKED",
    "ACCOUNT_ORDER_CREATION_LOCKED",
    "ACCOUNT_WITHDRAWAL_LOCKED",
    "ADMIN_CONFIGURE_DATA_MISSING",
    "ALIAS_INVALID",
    "AMOUNT_INVALID",
    "AMOUNT_MISSING",
    "CLIENT_CONFIGURE_DATA_MISSING",
    "CLIENT_EXTENSIONS_DATA_MISSING",
    "CLIENT_ORDER_COMMENT_INVALID",
    "CLIENT_ORDER_ID_ALREADY_EXISTS",
    "CLIENT_ORDER_ID_INVALID",
    "CLIENT_ORDER_TAG_INVALID",
    "CLIENT_TRADE_COMMENT_INVALID",
    "CLIENT_TRADE_ID_ALREADY_EXISTS",
    "CLIENT_TRADE_ID_INVALID",
    "CLIENT_TRADE_TAG_INVALID",
    "CLOSE_TRADE_PARTIAL_UNITS_MISSING",
    "CLOSE_TRADE_TYPE_MISSING",
    "CLOSE_TRADE_UNITS_EXCEED_TRADE_SIZE",
    "CLOSEOUT_POSITION_DOESNT_EXIST",
    "CLOSEOUT_POSITION_INCOMPLETE_SPECIFICATION",
    "CLOSEOUT_POSITION_PARTIAL_UNITS_MISSING",
    "CLOSEOUT_POSITION_REJECT",
    "CLOSEOUT_POSITION_UNITS_EXCEED_POSITION_SIZE",
    "FUNDING_REASON_MISSING",
    "INSTRUMENT_MISSING",
    "INSTRUMENT_NOT_TRADEABLE",
    "INSTRUMENT_PRICE_UNKNOWN",
    "INSTRUMENT_UNKNOWN",
    "INSUFFICIENT_FUNDS",
    "INSUFFICIENT_MARGIN",
    "INTERNAL_SERVER_ERROR",
    "INVALID_REISSUE_IMMEDIATE_PARTIAL_FILL",
    "MARGIN_RATE_INVALID",
    "MARGIN_RATE_WOULD_TRIGGER_CLOSEOUT",
    "MARGIN_RATE_WOULD_TRIGGER_MARGIN_CALL",
    "MARKUP_GROUP_ID_INVALID",
    "ORDER_DOESNT_EXIST",
    "ORDER_FILL_POSITION_ACTION_INVALID",
    "ORDER_FILL_POSITION_ACTION_MISSING",
    "ORDER_ID_UNSPECIFIED",
    "ORDER_IDENTIFIER_INCONSISTENCY",
    "ORDER_PARTIAL_FILL_OPTION_INVALID",
    "ORDER_PARTIAL_FILL_OPTION_MISSING",
    "ORDERS_ON_FILL_DUPLICATE_CLIENT_ORDER_IDS",
    "PENDING_ORDERS_ALLOWED_EXCEEDED",
    "POSITION_AGGREGATION_MODE_INVALID",
    "PRICE_BOUND_INVALID",
    "PRICE_BOUND_PRECISION_EXCEEDED",
    "PRICE_DISTANCE_INVALID",
    "PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "PRICE_DISTANCE_MINIMUM_NOT_MET",
    "PRICE_DISTANCE_MISSING",
    "PRICE_DISTANCE_PRECISION_EXCEEDED",
    "PRICE_INVALID",
    "PRICE_MISSING",
    "PRICE_PRECISION_EXCEEDED",
    "REPLACING_ORDER_INVALID",
    "REPLACING_TRADE_ID_INVALID",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_COMMENT_INVALID",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_INVALID",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_TAG_INVALID",
    "STOP_LOSS_ON_FILL_DISTANCE_INVALID",
    "STOP_LOSS_ON_FILL_DISTANCE_PRECISION_EXCEEDED",
    "STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "STOP_LOSS_ON_FILL_GTD_TIMESTAMP_MISSING",
    "STOP_LOSS_ON_FILL_GUARANTEED_LEVEL_RESTRICTION_EXCEEDED",
    "STOP_LOSS_ON_FILL_GUARANTEED_MINIMUM_DISTANCE_NOT_MET",
    "STOP_LOSS_ON_FILL_GUARANTEED_NOT_ALLOWED",
    "STOP_LOSS_ON_FILL_GUARANTEED_REQUIRED",
    "STOP_LOSS_ON_FILL_PRICE_AND_DISTANCE_BOTH_MISSING",
    "STOP_LOSS_ON_FILL_PRICE_AND_DISTANCE_BOTH_SPECIFIED",
    "STOP_LOSS_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "STOP_LOSS_ON_FILL_PRICE_INVALID",
    "STOP_LOSS_ON_FILL_PRICE_MISSING",
    "STOP_LOSS_ON_FILL_PRICE_PRECISION_EXCEEDED",
    "STOP_LOSS_ON_FILL_REQUIRED_FOR_PENDING_ORDER",
    "STOP_LOSS_ON_FILL_TIME_IN_FORCE_INVALID",
    "STOP_LOSS_ON_FILL_TIME_IN_FORCE_MISSING",
    "STOP_LOSS_ON_FILL_TRIGGER_CONDITION_INVALID",
    "STOP_LOSS_ON_FILL_TRIGGER_CONDITION_MISSING",
    "STOP_LOSS_ORDER_ALREADY_EXISTS",
    "STOP_LOSS_ORDER_GUARANTEED_HALTED_CREATE_VIOLATION",
    "STOP_LOSS_ORDER_GUARANTEED_HALTED_TIGHTEN_VIOLATION",
    "STOP_LOSS_ORDER_GUARANTEED_HEDGING_NOT_ALLOWED",
    "STOP_LOSS_ORDER_GUARANTEED_LEVEL_RESTRICTION_EXCEEDED",
    "STOP_LOSS_ORDER_GUARANTEED_MINIMUM_DISTANCE_NOT_MET",
    "STOP_LOSS_ORDER_GUARANTEED_NOT_ALLOWED",
    "STOP_LOSS_ORDER_GUARANTEED_PRICE_WITHIN_SPREAD",
    "STOP_LOSS_ORDER_GUARANTEED_REQUIRED",
    "STOP_LOSS_ORDER_NOT_CANCELABLE",
    "STOP_LOSS_ORDER_NOT_REPLACEABLE",
    "STOP_LOSS_ORDER_PRICE_AND_DISTANCE_BOTH_MISSING",
    "STOP_LOSS_ORDER_PRICE_AND_DISTANCE_BOTH_SPECIFIED",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_COMMENT_INVALID",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_ID_INVALID",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_TAG_INVALID",
    "TAKE_PROFIT_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "TAKE_PROFIT_ON_FILL_GTD_TIMESTAMP_MISSING",
    "TAKE_PROFIT_ON_FILL_PRICE_INVALID",
    "TAKE_PROFIT_ON_FILL_PRICE_MISSING",
    "TAKE_PROFIT_ON_FILL_PRICE_PRECISION_EXCEEDED",
    "TAKE_PROFIT_ON_FILL_TIME_IN_FORCE_INVALID",
    "TAKE_PROFIT_ON_FILL_TIME_IN_FORCE_MISSING",
    "TAKE_PROFIT_ON_FILL_TRIGGER_CONDITION_INVALID",
    "TAKE_PROFIT_ON_FILL_TRIGGER_CONDITION_MISSING",
    "TAKE_PROFIT_ORDER_ALREADY_EXISTS",
    "TIME_IN_FORCE_GTD_TIMESTAMP_IN_PAST",
    "TIME_IN_FORCE_GTD_TIMESTAMP_MISSING",
    "TIME_IN_FORCE_INVALID",
    "TIME_IN_FORCE_MISSING",
    "TRADE_DOESNT_EXIST",
    "TRADE_ID_UNSPECIFIED",
    "TRADE_IDENTIFIER_INCONSISTENCY",
    "TRADE_ON_FILL_CLIENT_EXTENSIONS_NOT_SUPPORTED",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_COMMENT_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_TAG_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "TRAILING_STOP_LOSS_ON_FILL_GTD_TIMESTAMP_MISSING",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_MINIMUM_NOT_MET",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_MISSING",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_PRECISION_EXCEEDED",
    "TRAILING_STOP_LOSS_ON_FILL_TIME_IN_FORCE_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_TIME_IN_FORCE_MISSING",
    "TRAILING_STOP_LOSS_ON_FILL_TRIGGER_CONDITION_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_TRIGGER_CONDITION_MISSING",
    "TRAILING_STOP_LOSS_ORDER_ALREADY_EXISTS",
    "TRAILING_STOP_LOSS_ORDERS_NOT_SUPPORTED",
    "TRIGGER_CONDITION_INVALID",
    "TRIGGER_CONDITION_MISSING",
    "UNITS_INVALID",
    "UNITS_LIMIT_EXCEEDED",
    "UNITS_MIMIMUM_NOT_MET",
    "UNITS_MISSING",
    "UNITS_PRECISION_EXCEEDED",
]
STOP_ORDER_REJECT_TRANSACTION_REJECT_REASON_VALUES: Set[
    StopOrderRejectTransactionRejectReason
] = {
    "ACCOUNT_CONFIGURATION_LOCKED",
    "ACCOUNT_DEPOSIT_LOCKED",
    "ACCOUNT_LOCKED",
    "ACCOUNT_NOT_ACTIVE",
    "ACCOUNT_ORDER_CANCEL_LOCKED",
    "ACCOUNT_ORDER_CREATION_LOCKED",
    "ACCOUNT_WITHDRAWAL_LOCKED",
    "ADMIN_CONFIGURE_DATA_MISSING",
    "ALIAS_INVALID",
    "AMOUNT_INVALID",
    "AMOUNT_MISSING",
    "CLIENT_CONFIGURE_DATA_MISSING",
    "CLIENT_EXTENSIONS_DATA_MISSING",
    "CLIENT_ORDER_COMMENT_INVALID",
    "CLIENT_ORDER_ID_ALREADY_EXISTS",
    "CLIENT_ORDER_ID_INVALID",
    "CLIENT_ORDER_TAG_INVALID",
    "CLIENT_TRADE_COMMENT_INVALID",
    "CLIENT_TRADE_ID_ALREADY_EXISTS",
    "CLIENT_TRADE_ID_INVALID",
    "CLIENT_TRADE_TAG_INVALID",
    "CLOSE_TRADE_PARTIAL_UNITS_MISSING",
    "CLOSE_TRADE_TYPE_MISSING",
    "CLOSE_TRADE_UNITS_EXCEED_TRADE_SIZE",
    "CLOSEOUT_POSITION_DOESNT_EXIST",
    "CLOSEOUT_POSITION_INCOMPLETE_SPECIFICATION",
    "CLOSEOUT_POSITION_PARTIAL_UNITS_MISSING",
    "CLOSEOUT_POSITION_REJECT",
    "CLOSEOUT_POSITION_UNITS_EXCEED_POSITION_SIZE",
    "FUNDING_REASON_MISSING",
    "INSTRUMENT_MISSING",
    "INSTRUMENT_NOT_TRADEABLE",
    "INSTRUMENT_PRICE_UNKNOWN",
    "INSTRUMENT_UNKNOWN",
    "INSUFFICIENT_FUNDS",
    "INSUFFICIENT_MARGIN",
    "INTERNAL_SERVER_ERROR",
    "INVALID_REISSUE_IMMEDIATE_PARTIAL_FILL",
    "MARGIN_RATE_INVALID",
    "MARGIN_RATE_WOULD_TRIGGER_CLOSEOUT",
    "MARGIN_RATE_WOULD_TRIGGER_MARGIN_CALL",
    "MARKUP_GROUP_ID_INVALID",
    "ORDER_DOESNT_EXIST",
    "ORDER_FILL_POSITION_ACTION_INVALID",
    "ORDER_FILL_POSITION_ACTION_MISSING",
    "ORDER_ID_UNSPECIFIED",
    "ORDER_IDENTIFIER_INCONSISTENCY",
    "ORDER_PARTIAL_FILL_OPTION_INVALID",
    "ORDER_PARTIAL_FILL_OPTION_MISSING",
    "ORDERS_ON_FILL_DUPLICATE_CLIENT_ORDER_IDS",
    "PENDING_ORDERS_ALLOWED_EXCEEDED",
    "POSITION_AGGREGATION_MODE_INVALID",
    "PRICE_BOUND_INVALID",
    "PRICE_BOUND_PRECISION_EXCEEDED",
    "PRICE_DISTANCE_INVALID",
    "PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "PRICE_DISTANCE_MINIMUM_NOT_MET",
    "PRICE_DISTANCE_MISSING",
    "PRICE_DISTANCE_PRECISION_EXCEEDED",
    "PRICE_INVALID",
    "PRICE_MISSING",
    "PRICE_PRECISION_EXCEEDED",
    "REPLACING_ORDER_INVALID",
    "REPLACING_TRADE_ID_INVALID",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_COMMENT_INVALID",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_INVALID",
    "STOP_LOSS_ON_FILL_CLIENT_ORDER_TAG_INVALID",
    "STOP_LOSS_ON_FILL_DISTANCE_INVALID",
    "STOP_LOSS_ON_FILL_DISTANCE_PRECISION_EXCEEDED",
    "STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "STOP_LOSS_ON_FILL_GTD_TIMESTAMP_MISSING",
    "STOP_LOSS_ON_FILL_GUARANTEED_LEVEL_RESTRICTION_EXCEEDED",
    "STOP_LOSS_ON_FILL_GUARANTEED_MINIMUM_DISTANCE_NOT_MET",
    "STOP_LOSS_ON_FILL_GUARANTEED_NOT_ALLOWED",
    "STOP_LOSS_ON_FILL_GUARANTEED_REQUIRED",
    "STOP_LOSS_ON_FILL_PRICE_AND_DISTANCE_BOTH_MISSING",
    "STOP_LOSS_ON_FILL_PRICE_AND_DISTANCE_BOTH_SPECIFIED",
    "STOP_LOSS_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "STOP_LOSS_ON_FILL_PRICE_INVALID",
    "STOP_LOSS_ON_FILL_PRICE_MISSING",
    "STOP_LOSS_ON_FILL_PRICE_PRECISION_EXCEEDED",
    "STOP_LOSS_ON_FILL_REQUIRED_FOR_PENDING_ORDER",
    "STOP_LOSS_ON_FILL_TIME_IN_FORCE_INVALID",
    "STOP_LOSS_ON_FILL_TIME_IN_FORCE_MISSING",
    "STOP_LOSS_ON_FILL_TRIGGER_CONDITION_INVALID",
    "STOP_LOSS_ON_FILL_TRIGGER_CONDITION_MISSING",
    "STOP_LOSS_ORDER_ALREADY_EXISTS",
    "STOP_LOSS_ORDER_GUARANTEED_HALTED_CREATE_VIOLATION",
    "STOP_LOSS_ORDER_GUARANTEED_HALTED_TIGHTEN_VIOLATION",
    "STOP_LOSS_ORDER_GUARANTEED_HEDGING_NOT_ALLOWED",
    "STOP_LOSS_ORDER_GUARANTEED_LEVEL_RESTRICTION_EXCEEDED",
    "STOP_LOSS_ORDER_GUARANTEED_MINIMUM_DISTANCE_NOT_MET",
    "STOP_LOSS_ORDER_GUARANTEED_NOT_ALLOWED",
    "STOP_LOSS_ORDER_GUARANTEED_PRICE_WITHIN_SPREAD",
    "STOP_LOSS_ORDER_GUARANTEED_REQUIRED",
    "STOP_LOSS_ORDER_NOT_CANCELABLE",
    "STOP_LOSS_ORDER_NOT_REPLACEABLE",
    "STOP_LOSS_ORDER_PRICE_AND_DISTANCE_BOTH_MISSING",
    "STOP_LOSS_ORDER_PRICE_AND_DISTANCE_BOTH_SPECIFIED",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_COMMENT_INVALID",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_ID_INVALID",
    "TAKE_PROFIT_ON_FILL_CLIENT_ORDER_TAG_INVALID",
    "TAKE_PROFIT_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "TAKE_PROFIT_ON_FILL_GTD_TIMESTAMP_MISSING",
    "TAKE_PROFIT_ON_FILL_PRICE_INVALID",
    "TAKE_PROFIT_ON_FILL_PRICE_MISSING",
    "TAKE_PROFIT_ON_FILL_PRICE_PRECISION_EXCEEDED",
    "TAKE_PROFIT_ON_FILL_TIME_IN_FORCE_INVALID",
    "TAKE_PROFIT_ON_FILL_TIME_IN_FORCE_MISSING",
    "TAKE_PROFIT_ON_FILL_TRIGGER_CONDITION_INVALID",
    "TAKE_PROFIT_ON_FILL_TRIGGER_CONDITION_MISSING",
    "TAKE_PROFIT_ORDER_ALREADY_EXISTS",
    "TIME_IN_FORCE_GTD_TIMESTAMP_IN_PAST",
    "TIME_IN_FORCE_GTD_TIMESTAMP_MISSING",
    "TIME_IN_FORCE_INVALID",
    "TIME_IN_FORCE_MISSING",
    "TRADE_DOESNT_EXIST",
    "TRADE_ID_UNSPECIFIED",
    "TRADE_IDENTIFIER_INCONSISTENCY",
    "TRADE_ON_FILL_CLIENT_EXTENSIONS_NOT_SUPPORTED",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_COMMENT_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_ID_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_CLIENT_ORDER_TAG_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_GTD_TIMESTAMP_IN_PAST",
    "TRAILING_STOP_LOSS_ON_FILL_GTD_TIMESTAMP_MISSING",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_MAXIMUM_EXCEEDED",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_MINIMUM_NOT_MET",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_MISSING",
    "TRAILING_STOP_LOSS_ON_FILL_PRICE_DISTANCE_PRECISION_EXCEEDED",
    "TRAILING_STOP_LOSS_ON_FILL_TIME_IN_FORCE_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_TIME_IN_FORCE_MISSING",
    "TRAILING_STOP_LOSS_ON_FILL_TRIGGER_CONDITION_INVALID",
    "TRAILING_STOP_LOSS_ON_FILL_TRIGGER_CONDITION_MISSING",
    "TRAILING_STOP_LOSS_ORDER_ALREADY_EXISTS",
    "TRAILING_STOP_LOSS_ORDERS_NOT_SUPPORTED",
    "TRIGGER_CONDITION_INVALID",
    "TRIGGER_CONDITION_MISSING",
    "UNITS_INVALID",
    "UNITS_LIMIT_EXCEEDED",
    "UNITS_MIMIMUM_NOT_MET",
    "UNITS_MISSING",
    "UNITS_PRECISION_EXCEEDED",
}


def check_stop_order_reject_transaction_reject_reason(
    value: str,
) -> StopOrderRejectTransactionRejectReason:
    if value in STOP_ORDER_REJECT_TRANSACTION_REJECT_REASON_VALUES:
        return cast(StopOrderRejectTransactionRejectReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_ORDER_REJECT_TRANSACTION_REJECT_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="DynamicOrderState")


@dataclasses.dataclass
class DynamicOrderState:
    """The dynamic state of an Order. This is only relevant to TrailingStopLoss Orders, as no other Order type has dynamic
    state.

        Attributes:
            id (Optional[str]): The Order's ID.
            trailing_stop_value (Optional[str]): The Order's calculated trailing stop value.
            trigger_distance (Optional[str]): The distance between the Trailing Stop Loss Order's trailingStopValue and
                the current Market Price. This represents the distance (in price units) of the Order from a triggering price. If
                the distance could not be determined, this value will not be set.
            is_trigger_distance_exact (Optional[bool]): True if an exact trigger distance could be calculated. If false,
                it means the provided trigger distance is a best estimate. If the distance could not be determined, this value
                will not be set."""

    id: Optional[str]
    trailing_stop_value: Optional[str]
    trigger_distance: Optional[str]
    is_trigger_distance_exact: Optional[bool]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        trailing_stop_value = d.pop("trailingStopValue", None)
        trigger_distance = d.pop("triggerDistance", None)
        is_trigger_distance_exact = d.pop("isTriggerDistanceExact", None)
        dynamic_order_state = cls(
            id=id,
            trailing_stop_value=trailing_stop_value,
            trigger_distance=trigger_distance,
            is_trigger_distance_exact=is_trigger_distance_exact,
        )
        dynamic_order_state.additional_properties = d
        return dynamic_order_state

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .account_changes import AccountChanges
from .account_changes_state import AccountChangesState
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetAccountChangesResponse200")


@dataclasses.dataclass
class GetAccountChangesResponse200:
    """Attributes:
    changes (Optional[AccountChanges]): An AccountChanges Object is used to represent the changes to an
        Account's Orders, Trades and Positions since a specified Account TransactionID in the past.
    state (Optional[AccountChangesState]): An AccountState Object is used to represent an Account's current
        price-dependent state. Price-dependent Account state is dependent on OANDA's current Prices, and includes things
        like unrealized PL, NAV and Trailing Stop Loss Order state.
    last_transaction_id (Optional[str]): The ID of the last Transaction created for the Account.  This
        Transaction ID should be used for future poll requests, as the client has already observed all changes up to and
        including it."""

    changes: Optional["AccountChanges"]
    state: Optional["AccountChangesState"]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .account_changes import AccountChanges
        from .account_changes_state import AccountChangesState

        d = src_dict.copy()
        _changes = d.pop("changes", None)
        changes: Optional[AccountChanges]
        if _changes is None:
            changes = None
        else:
            changes = AccountChanges.from_dict(_changes)
        _state = d.pop("state", None)
        state: Optional[AccountChangesState]
        if _state is None:
            state = None
        else:
            state = AccountChangesState.from_dict(_state)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_account_changes_response_200 = cls(
            changes=changes, state=state, last_transaction_id=last_transaction_id
        )
        get_account_changes_response_200.additional_properties = d
        return get_account_changes_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order import Order
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetOrderResponse200")


@dataclasses.dataclass
class GetOrderResponse200:
    """Attributes:
    order (Optional[Order]): The base Order definition specifies the properties that are common to all Orders.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    order: Optional["Order"]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .order import Order

        d = src_dict.copy()
        _order = d.pop("order", None)
        order: Optional[Order]
        if _order is None:
            order = None
        else:
            order = Order.from_dict(_order)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_order_response_200 = cls(
            order=order, last_transaction_id=last_transaction_id
        )
        get_order_response_200.additional_properties = d
        return get_order_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_configure_transaction import ClientConfigureTransaction
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="ConfigureAccountResponse200")


@dataclasses.dataclass
class ConfigureAccountResponse200:
    """Attributes:
    client_configure_transaction (Optional[ClientConfigureTransaction]): A ClientConfigureTransaction represents
        the configuration of an Account by a client.
    last_transaction_id (Optional[str]): The ID of the last Transaction created for the Account."""

    client_configure_transaction: Optional["ClientConfigureTransaction"]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_configure_transaction import ClientConfigureTransaction

        d = src_dict.copy()
        _client_configure_transaction = d.pop("clientConfigureTransaction", None)
        client_configure_transaction: Optional[ClientConfigureTransaction]
        if _client_configure_transaction is None:
            client_configure_transaction = None
        else:
            client_configure_transaction = ClientConfigureTransaction.from_dict(
                _client_configure_transaction
            )
        last_transaction_id = d.pop("lastTransactionID", None)
        configure_account_response_200 = cls(
            client_configure_transaction=client_configure_transaction,
            last_transaction_id=last_transaction_id,
        )
        configure_account_response_200.additional_properties = d
        return configure_account_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .position import Position
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetPositionResponse200")


@dataclasses.dataclass
class GetPositionResponse200:
    """Attributes:
    position (Optional[Position]): The specification of a Position within an Account.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    position: Optional["Position"]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .position import Position

        d = src_dict.copy()
        _position = d.pop("position", None)
        position: Optional[Position]
        if _position is None:
            position = None
        else:
            position = Position.from_dict(_position)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_position_response_200 = cls(
            position=position, last_transaction_id=last_transaction_id
        )
        get_position_response_200.additional_properties = d
        return get_position_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .stop_loss_order import StopLossOrder
from .take_profit_order import TakeProfitOrder
from .trade_state import TradeState
from .trade_state import check_trade_state
from .trailing_stop_loss_order import TrailingStopLossOrder
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="Trade")


@dataclasses.dataclass
class Trade:
    """The specification of a Trade within an Account. This includes the full representation of the Trade's dependent
    Orders in addition to the IDs of those Orders.

        Attributes:
            id (Optional[str]): The Trade's identifier, unique within the Trade's Account.
            instrument (Optional[str]): The Trade's Instrument.
            price (Optional[str]): The execution price of the Trade.
            open_time (Optional[str]): The date/time when the Trade was opened.
            state (Optional[TradeState]): The current state of the Trade.
            initial_units (Optional[str]): The initial size of the Trade. Negative values indicate a short Trade, and
                positive values indicate a long Trade.
            initial_margin_required (Optional[str]): The margin required at the time the Trade was created. Note, this
                is the 'pure' margin required, it is not the 'effective' margin used that factors in the trade risk if a GSLO is
                attached to the trade.
            current_units (Optional[str]): The number of units currently open for the Trade. This value is reduced to
                0.0 as the Trade is closed.
            realized_pl (Optional[str]): The total profit/loss realized on the closed portion of the Trade.
            unrealized_pl (Optional[str]): The unrealized profit/loss on the open portion of the Trade.
            margin_used (Optional[str]): Margin currently used by the Trade.
            average_close_price (Optional[str]): The average closing price of the Trade. Only present if the Trade has
                been closed or reduced at least once.
            closing_transaction_i_ds (Optional[List[str]]): The IDs of the Transactions that have closed portions of
                this Trade.
            financing (Optional[str]): The financing paid/collected for this Trade.
            close_time (Optional[str]): The date/time when the Trade was fully closed. Only provided for Trades whose
                state is CLOSED.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            take_profit_order (Optional[TakeProfitOrder]): A TakeProfitOrder is an order that is linked to an open Trade
                and created with a price threshold. The Order will be filled (closing the Trade) by the first price that is
                equal to or better than the threshold. A TakeProfitOrder cannot be used to open a new Position.
            stop_loss_order (Optional[StopLossOrder]): A StopLossOrder is an order that is linked to an open Trade and
                created with a price threshold. The Order will be filled (closing the Trade) by the first price that is equal to
                or worse than the threshold. A StopLossOrder cannot be used to open a new Position.
            trailing_stop_loss_order (Optional[TrailingStopLossOrder]): A TrailingStopLossOrder is an order that is
                linked to an open Trade and created with a price distance. The price distance is used to calculate a trailing
                stop value for the order that is in the losing direction from the market price at the time of the order's
                creation. The trailing stop value will follow the market price as it moves in the winning direction, and the
                order will filled (closing the Trade) by the first price that is equal to or worse than the trailing stop value.
                A TrailingStopLossOrder cannot be used to open a new Position."""

    id: Optional[str]
    instrument: Optional[str]
    price: Optional[str]
    open_time: Optional[str]
    state: Optional[TradeState]
    initial_units: Optional[str]
    initial_margin_required: Optional[str]
    current_units: Optional[str]
    realized_pl: Optional[str]
    unrealized_pl: Optional[str]
    margin_used: Optional[str]
    average_close_price: Optional[str]
    closing_transaction_i_ds: Optional[List[str]]
    financing: Optional[str]
    close_time: Optional[str]
    client_extensions: Optional["ClientExtensions"]
    take_profit_order: Optional["TakeProfitOrder"]
    stop_loss_order: Optional["StopLossOrder"]
    trailing_stop_loss_order: Optional["TrailingStopLossOrder"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .trailing_stop_loss_order import TrailingStopLossOrder
        from .stop_loss_order import StopLossOrder
        from .client_extensions import ClientExtensions
        from .take_profit_order import TakeProfitOrder

        d = src_dict.copy()
        id = d.pop("id", None)
        instrument = d.pop("instrument", None)
        price = d.pop("price", None)
        open_time = d.pop("openTime", None)
        _state = d.pop("state", None)
        state: Optional[TradeState]
        if _state is None:
            state = None
        else:
            state = check_trade_state(_state)
        initial_units = d.pop("initialUnits", None)
        initial_margin_required = d.pop("initialMarginRequired", None)
        current_units = d.pop("currentUnits", None)
        realized_pl = d.pop("realizedPL", None)
        unrealized_pl = d.pop("unrealizedPL", None)
        margin_used = d.pop("marginUsed", None)
        average_close_price = d.pop("averageClosePrice", None)
        closing_transaction_i_ds = cast(List[str], d.pop("closingTransactionIDs", None))
        financing = d.pop("financing", None)
        close_time = d.pop("closeTime", None)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _take_profit_order = d.pop("takeProfitOrder", None)
        take_profit_order: Optional[TakeProfitOrder]
        if _take_profit_order is None:
            take_profit_order = None
        else:
            take_profit_order = TakeProfitOrder.from_dict(_take_profit_order)
        _stop_loss_order = d.pop("stopLossOrder", None)
        stop_loss_order: Optional[StopLossOrder]
        if _stop_loss_order is None:
            stop_loss_order = None
        else:
            stop_loss_order = StopLossOrder.from_dict(_stop_loss_order)
        _trailing_stop_loss_order = d.pop("trailingStopLossOrder", None)
        trailing_stop_loss_order: Optional[TrailingStopLossOrder]
        if _trailing_stop_loss_order is None:
            trailing_stop_loss_order = None
        else:
            trailing_stop_loss_order = TrailingStopLossOrder.from_dict(
                _trailing_stop_loss_order
            )
        trade = cls(
            id=id,
            instrument=instrument,
            price=price,
            open_time=open_time,
            state=state,
            initial_units=initial_units,
            initial_margin_required=initial_margin_required,
            current_units=current_units,
            realized_pl=realized_pl,
            unrealized_pl=unrealized_pl,
            margin_used=margin_used,
            average_close_price=average_close_price,
            closing_transaction_i_ds=closing_transaction_i_ds,
            financing=financing,
            close_time=close_time,
            client_extensions=client_extensions,
            take_profit_order=take_profit_order,
            stop_loss_order=stop_loss_order,
            trailing_stop_loss_order=trailing_stop_loss_order,
        )
        trade.additional_properties = d
        return trade

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="Transaction")


@dataclasses.dataclass
class Transaction:
    """The base Transaction specification. Specifies properties that are common between all Transaction.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction."""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
        )
        transaction.additional_properties = d
        return transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="ClientExtensions")


@dataclasses.dataclass
class ClientExtensions:
    """A ClientExtensions object allows a client to attach a clientID, tag and comment to Orders and Trades in their
    Account.  Do not set, modify, or delete this field if your account is associated with MT4.

        Attributes:
            id (Optional[str]): The Client ID of the Order/Trade
            tag (Optional[str]): A tag associated with the Order/Trade
            comment (Optional[str]): A comment associated with the Order/Trade"""

    id: Optional[str]
    tag: Optional[str]
    comment: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        tag = d.pop("tag", None)
        comment = d.pop("comment", None)
        client_extensions = cls(id=id, tag=tag, comment=comment)
        client_extensions.additional_properties = d
        return client_extensions

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

TrailingStopLossOrderTransactionReason = Literal[
    "CLIENT_ORDER", "ON_FILL", "REPLACEMENT"
]

TRAILING_STOP_LOSS_ORDER_TRANSACTION_REASON_VALUES: Set[
    TrailingStopLossOrderTransactionReason
] = {
    "CLIENT_ORDER",
    "ON_FILL",
    "REPLACEMENT",
}


def check_trailing_stop_loss_order_transaction_reason(
    value: str,
) -> TrailingStopLossOrderTransactionReason:
    if value in TRAILING_STOP_LOSS_ORDER_TRANSACTION_REASON_VALUES:
        return cast(TrailingStopLossOrderTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRAILING_STOP_LOSS_ORDER_TRANSACTION_REASON_VALUES!r}"
    )
